import json
import subprocess
import requests
import logging
from voucher import __version__ as version

headers = {
    "Accept": "application/json, text/plain, */*",
    "User-Agent": f"wifi-voucher/{version} https://git.rre.nu/jonas/wifi-voucher",
}


def print_voucher(args, voucher):
    logging.debug(f"Printing voucher: {voucher[0:5]}-{voucher[5:]}")
    validstr = (
        f"Valid for {args.unifi_voucher_expire_number} {args.unifi_voucher_expire_unit}"
    )
    if args.unifi_voucher_expire_number != 1:
        validstr = validstr + "s"
    lpr = subprocess.Popen(
        ["/usr/bin/lpr", "-P", args.printer],
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    lpr.stdin.write(
        f"GoHome Guest Wifi \n\n{voucher[0:5]}-{voucher[5:]}\n\n{validstr}\n".encode()
    )
    output, error = lpr.communicate()
    if error:
        logging.error(error.decode("utf-8").strip())


def create_voucher(args):
    """
    Creates a wifi-voucher
    """
    logging.debug("Creating voucher")
    session = requests.Session()
    session.verify = not args.unifi_insecure

    # Login
    payload = {
        "username": args.unifi_user,
        "password": args.unifi_password,
        "for_hotspot": True,
        "remember": False,
        "site_name": args.unifi_site,
    }
    try:
        response = session.post(
            f"https://{args.unifi_host}:{args.unifi_port}/api/login",
            data=json.dumps(payload),
            headers=headers,
        )
    except requests.exceptions.RequestException as err:
        logging.error(err)
        return []
    if response.status_code != 200:
        logging.error(
            f"Got response code '{response.status_code}' from https://{args.unifi_host}:{args.unifi_port}"
        )
        return []
    try:
        if response.json()["meta"]["rc"] != "ok":
            logging.error(f"Error in response from unifi API: {response.json()}")
            return []
    except KeyError:
        logging.error("Got invalid json response from Unifi controller during login")
        return []

    # create voucher
    payload = {}
    payload["cmd"] = "create-voucher"
    payload["note"] = f"wifi-voucher: {version}"
    payload["quota"] = args.unifi_voucher_quota
    payload["expire_unit"] = 1440
    if args.unifi_voucher_expire_unit in ["minute", "hours"]:
        payload["expire_number"] = 1
        if args.unifi_voucher_expire_unit == "minute":
            payload["expire"] = args.unifi_voucher_expire_number
        else:
            payload["expire"] = args.unifi_voucher_expire_number * 60
    else:
        payload["expire"] = "custom"
        if args.unifi_voucher_expire_unit == "day":
            payload["expire_number"] = args.unifi_voucher_expire_number
        elif args.unifi_voucher_expire_unit == "week":
            payload["expire_number"] = args.unifi_voucher_expire_number * 7
        elif args.unifi_voucher_expire_unit == "year":
            payload["expire_number"] = args.unifi_voucher_expire_number * 364

    payload["n"] = args.unifi_voucher_number
    if args.unifi_voucher_limit_byte:
        payload["byte"] = args.unifi_voucher_limit_byte
    if args.unifi_voucher_limit_up:
        payload["up"] = args.unifi_voucher_limit_up
    if args.unifi_voucher_limit_down:
        payload["down"] = args.unifi_voucher_limit_down

    try:
        response = session.post(
            f"https://{args.unifi_host}:{args.unifi_port}/api/s/{args.unifi_site}/cmd/hotspot",
            data=json.dumps(payload),
            headers=headers,
        )
    except requests.exceptions.RequestException as err:
        logging.error(err)
        return []
    if response.status_code != 200:
        logging.error(
            f"Got response code '{response.status_code}' from https://{args.unifi_host}:{args.unifi_port}"
        )
        return []
    try:
        if response.json()["meta"]["rc"] != "ok":
            logging.error(f"Error in response from unifi API: {response.json()}")
            return []
    except KeyError:
        logging.error(
            "Got invalid json response from Unifi controller during voucher creation"
        )
        return []
    creation_time_list = response.json()["data"]

    # Get created voucher.
    response = session.get(
        f"https://{args.unifi_host}:{args.unifi_port}/api/s/{args.unifi_site}/stat/voucher",
        headers=headers,
    )
    voucher_codes = []
    for voucher in response.json()["data"]:
        for item in creation_time_list:
            if voucher["create_time"] == item["create_time"]:
                voucher_codes.append(voucher["code"])

    return voucher_codes
